@@ -6,7 +6,7 @@ use strict;
 use MIME::Base64;
 use Exporter 'import';
 
-our $VERSION = '0.3';
+our $VERSION = '0.5';
 
 our %EXPORT_TAGS = (
     localStorage => [
@@ -85,62 +85,71 @@ our (
 use mod_perl;
 
 BEGIN {
-    if ( exists $ENV{MOD_PERL} ) {
-        if ( $ENV{MOD_PERL_API_VERSION} >= 2 ) {
-            *MP = sub { 2 };
+    if( exists $ENV{MOD_PERL} ) {
+        if( $ENV{MOD_PERL_API_VERSION} >= 2 ) {
+            *MP = sub{2};
         }
         else {
-            *MP = sub { 1 };
+            *MP = sub{1};
         }
     }
     else {
-        *MP = sub { 0 };
+        *MP = sub{0};
     }
-    if ( MP() == 2 ) {
-
-        #eval{
-        #    use Apache2::RequestRec;
-        #    use Apache2::Log;
-        #    use Apache2::Const -compile => qw(:common :log FORBIDDEN);
-        #};
-        #die $@ if($@);
+    if (MP() == 2) {
+	#eval{
+	#    use Apache2::RequestRec;
+	#    use Apache2::Log;
+	#    use Apache2::Const -compile => qw(:common :log FORBIDDEN);
+	#};
+	#die $@ if($@);
         require Apache2::RequestRec;
-        Apache2::RequestRec->import();
-
+	Apache2::RequestRec->import();
         #require Apache2::RequestIO;
-        require Apache2::Log;
+	require Apache2::Log;
         require Apache2::Const;
-
         #Apache2::Const->import('-compile', 'FORBIDDEN');
-        Apache2::Const->import( '-compile', qw(:common :log) );
-        *FORBIDDEN    = \&Apache2::Const::FORBIDDEN;
-        *REDIRECT     = \&Apache2::Const::REDIRECT;
-        *OK           = \&Apache2::Const::OK;
-        *DECLINED     = \&Apache2::Const::DECLINED;
-        *DONE         = \&Apache2::Const::DONE;
-        *SERVER_ERROR = \&Apache2::Const::SERVER_ERROR;
+	Apache2::Const->import('-compile', qw(:common :log));
+	*FORBIDDEN = \&Apache2::Const::FORBIDDEN;
+	*REDIRECT = \&Apache2::Const::REDIRECT;
+	*OK = \&Apache2::Const::OK;
+	*DECLINED = \&Apache2::Const::DECLINED;
+	*DONE = \&Apache2::Const::DONE;
+	*SERVER_ERROR = \&Apache2::Const::SERVER_ERROR;
         require Apache2::compat;
         Apache2::compat->import();
+	require threads::shared;
+	threads::shared::share($locationRegexp);
+	threads::shared::share($locationCondition);
+	threads::shared::share($defaultCondition);
+	threads::shared::share($forgeHeaders);
+	threads::shared::share($locationCount);
+	threads::shared::share($cookieName);
+	threads::shared::share($portal);
+	threads::shared::share($globalStorage);
+	threads::shared::share($globalStorageOptions);
+	threads::shared::share($localStorage);
+	threads::shared::share($localStorageOptions);
+	threads::shared::share($whatToTrace);
+	threads::shared::share($https);
+	threads::shared::share($refLocalStorage);
     }
     else {
         require Apache;
         require Apache::Log;
         require Apache::Constants;
         Apache::Constants->import(':common');
-        Apache::Constants->import(':response');
+	Apache::Constants->import(':response');
     }
-    *handler = ( MP() == 2 ) ? \&handler_mp2 : \&handler_mp1;
+    *handler = MP() ? \&handler_mp2 : \&handler_mp1;
 }
 
-sub handler_mp1 ($$) { &run }
-
-sub handler_mp2 : method {
-    &run;
-}
+sub handler_mp1 ($$)     { shift->run(@_) }
+sub handler_mp2 : method { shift->run(@_) }
 
 sub lmLog($$$) {
-    my ( $class, $mess, $level ) = @_;
-    if ( MP() == 2 ) {
+    my($class,$mess,$level) = @_;
+    if(MP()==2) {
         Apache2::ServerRec->log->$level($mess);
     }
     else {
@@ -149,64 +158,63 @@ sub lmLog($$$) {
 }
 
 sub regRemoteIp {
-    my ( $class, $str ) = @_;
-    if ( MP() == 2 ) {
-        $str =~
-          s/\$datas->\{ip\}/\$apacheRequest->connection->remote_addr->ip_get/g;
-    }
-    else {
-        $str =~ s/\$datas->\{ip\}/\$apacheRequest->connection->remote_ip/g;
-    }
+    my($class,$str) = @_;
+    #if(MP()==2) {
+	#$str =~ s/\$datas->\{ip\}/\$apacheRequest->connection->remote_addr->ip_get/g;
+    #}
+    #else {
+	$str =~ s/\$datas->\{ip\}/\$apacheRequest->connection->remote_ip/g;
+    #}
     return $str;
 }
 
 sub lmSetHeaderIn {
-    my ( $r, $h, $v ) = @_;
-    if ( MP() == 2 ) {
-        return $r->headers_in->set( $h => $v );
+    my($r,$h,$v) = @_;
+    if(MP()==2) {
+        return $r->headers_in->set($h => $v);
     }
     else {
-        return $r->header_in( $h => $v );
+	return $r->header_in($h => $v);
     }
 }
 
 sub lmHeaderIn {
-    my ( $r, $h, $v ) = @_;
-    if ( MP() == 2 ) {
+    my($r,$h,$v) = @_;
+    if(MP()==2) {
         return $r->headers_in->{$h};
     }
     else {
-        return $r->header_in($h);
+	return $r->header_in($h);
     }
 }
 
 sub lmSetErrHeaderOut {
-    my ( $r, $h, $v ) = @_;
-    if ( MP() == 2 ) {
-        return $r->err_headers_out->set( $h => $v );
+    my($r,$h,$v) = @_;
+    if(MP()==2) {
+        return $r->err_headers_out->set($h => $v);
     }
     else {
-        return $r->header_out( $h => $v );
+	return $r->header_out($h => $v);
     }
 }
 
 sub lmSetHeaderOut {
-    my ( $r, $h, $v ) = @_;
-    if ( MP() == 2 ) {
-        return $r->headers_out->set( $h => $v );
+    my($r,$h,$v) = @_;
+    if(MP()==2) {
+        return $r->headers_out->set($h => $v);
     }
     else {
-        return $r->header_out( $h => $v );
+	return $r->header_out($h => $v);
     }
 }
 
 sub lmHeaderOut {
-    my ( $r, $h, $v ) = @_;
-    if ( MP() == 2 ) {
+    my($r,$h,$v) = @_;
+    if(MP()==2) {
         return $r->headers_out->{$h};
     }
     else {
-        return $r->header_out($h);
+	return $r->header_out($h);
     }
 }
 
@@ -234,39 +242,33 @@ sub localInit($$) {
 
         eval "use $localStorage;";
         die("Unable to load $localStorage") if ($@);
-
-        # At each Apache (re)start, we've to clear the cache to avoid living
-        # with old datas
-        eval '$refLocalStorage = new '
-          . $localStorage
-          . '($localStorageOptions);';
-        if ( defined $refLocalStorage ) {
-            $refLocalStorage->clear();
-        }
-        else {
-            $class->lmLog( "Unable to clear local cache: $@", 'error' );
-        }
-
-    # We don't initialise local storage in the "init" subroutine because it can
-    # be used at the starting of Apache and so with the "root" privileges. Local
-    # Storage is also initialized just after Apache's fork and privilege lost.
-        no strict;
-
-     # Local storage is cleaned after giving the content of the page to increase
-     # performances.
-        if ( MP() == 2 ) {
-            Apache->push_handlers( PerlChildInitHandler =>
-                  sub { $class->initLocalStorage( $_[1], $_[0] ) } );
-            Apache->push_handlers(
-                PerlCleanupHandler => sub { $class->cleanLocalStorage(@_) } );
-        }
-        else {
-            Apache->push_handlers(
-                PerlChildInitHandler => sub { $class->initLocalStorage(@_) } );
-            Apache->push_handlers(
-                PerlCleanupHandler => sub { $class->cleanLocalStorage(@_) } );
-        }
+	# At each Apache (re)start, we've to clear the cache to avoid living
+	# with old datas
+	eval '$refLocalStorage = new '
+	  . $localStorage
+	  . '($localStorageOptions);';
+	if ( defined $refLocalStorage ) {
+	    $refLocalStorage->clear();
+	}
+	else {
+	    $class->lmLog("Unable to clear local cache: $@",'error');
+	}
     }
+        # We don't initialise local storage in the "init" subroutine because it can
+        # be used at the starting of Apache and so with the "root" privileges. Local
+        # Storage is also initialized just after Apache's fork and privilege lost.
+
+        # Local storage is cleaned after giving the content of the page to increase
+        # performances.
+	no strict;
+	if(MP()==2) {
+	    Apache->push_handlers( PerlChildInitHandler => sub { return $class->initLocalStorage($_[1], $_[0]); } );
+	    Apache->push_handlers( PerlCleanupHandler => sub { return $class->cleanLocalStorage(@_); } );
+	}
+	else {
+	    Apache->push_handlers( PerlChildInitHandler => sub { return $class->initLocalStorage(@_); } );
+	    Apache->push_handlers( PerlCleanupHandler => sub { return $class->cleanLocalStorage(@_); } );
+	}
 }
 
 # Global initialization process :
@@ -365,14 +367,13 @@ sub forgeHeadersInit {
     }
     foreach ( keys %tmp ) {
         $tmp{$_} =~ s/\$(\w+)/\$datas->{$1}/g;
-        $tmp{$_} = $class->regRemoteIp( $tmp{$_} );
+	$tmp{$_} = $class->regRemoteIp($tmp{$_});
     }
 
     my $sub;
     foreach ( keys %tmp ) {
         $sub .=
-          "lmSetHeaderIn(\$apacheRequest,'$_' => join('',split(/[\\r\\n]+/,"
-          . $tmp{$_} . ")));";
+          "lmSetHeaderIn(\$apacheRequest,'$_' => join('',split(/[\\r\\n]+/," . $tmp{$_} . ")));";
     }
     $sub = "\$forgeHeaders = sub {$sub};";
     eval "$sub";
@@ -396,24 +397,21 @@ sub grant {
 
 # forbidden : used to reject non authorizated requests
 sub forbidden {
-    my $class = shift;
+    my $class=shift;
 
     # We use Apache::Log here
-    $class->lmLog(
-        'The user "'
+    $class->lmLog( 'The user "'
           . $datas->{$whatToTrace}
           . '" was reject when he tried to access to '
-          . $_[1],
-        'notice'
-    );
+          . $_[1], 'notice' );
     return FORBIDDEN;
 }
 
 # hideCookie : hide Lemonldap cookie to the protected application
 sub hideCookie {
-    my $tmp = lmHeaderIn( $apacheRequest, 'Cookie' );
+    my $tmp = lmHeaderIn($apacheRequest,'Cookie');
     $tmp =~ s/$cookieName[^;]*;?//o;
-    lmSetHeaderIn( $apacheRequest, 'Cookie' => $tmp );
+    lmSetHeaderIn($apacheRequest, 'Cookie' => $tmp );
 }
 
 # Redirect non-authenticated users to the portal
@@ -425,17 +423,8 @@ sub goToPortal() {
           . $apacheRequest->get_server_name()
           . $url );
     $urlc_init =~ s/[\n\s]//g;
-    $class->lmLog(
-        "Redirect "
-          . $apacheRequest->connection->remote_ip
-          . " to portal (url was $url)",
-        'debug'
-    );
-    print STDERR "DEBUG a enlever\n";
-    use Data::Dumper;
-    print STDERR Dumper($portal);
-    $apacheRequest->headers_out->set( 'Location' => "$portal?url=$urlc_init" );
-
+    $class->lmLog( "Redirect " . $apacheRequest->connection->remote_ip . " to portal (url was $url)",'debug' );
+    $apacheRequest->headers_out->set('Location' => "$portal?url=$urlc_init" );
     #lmSetErrHeaderOut($apacheRequest, Location => "$portal?url=$urlc_init" );
     return REDIRECT;
 }
@@ -452,10 +441,7 @@ sub run ($$) {
     # AUTHENTICATION
     # I - recover the cookie
     my $id;
-    unless ( ($id) =
-        ( lmHeaderIn( $apacheRequest, 'Cookie' ) =~ /$cookieName=([^; ]+);?/o )
-      )
-    {
+    unless ( ($id) = ( lmHeaderIn($apacheRequest,'Cookie') =~ /$cookieName=([^; ]+);?/o ) ) {
         $class->lmLog( "$class: No cookie found", 'info' );
         return $class->goToPortal($uri);
     }
@@ -474,8 +460,7 @@ sub run ($$) {
             if ($@) {
 
                 # The cookie isn't yet available
-                $class->lmLog( "The cookie $id isn't yet available: $@",
-                    'info' );
+                $class->lmLog( "The cookie $id isn't yet available: $@", 'info');
                 return $class->goToPortal($uri);
             }
             $datas->{$_} = $h{$_} foreach ( keys %h );
@@ -490,12 +475,7 @@ sub run ($$) {
 
     # AUTHORIZATION
     return $class->forbidden($uri) unless ( $class->grant($uri) );
-    $class->lmLog(
-        "User "
-          . $datas->{$whatToTrace}
-          . " was authorizated to access to $uri",
-        'debug'
-    );
+    $class->lmLog( "User " . $datas->{$whatToTrace} . " was authorizated to access to $uri", 'debug' );
 
     # ACCOUNTING
     # 1 - Inform Apache
@@ -515,13 +495,13 @@ sub sendHeaders {
 }
 
 sub initLocalStorage {
-    my ( $class, $r ) = @_;
+    my($class,$r) = @_;
     if ( $localStorage and not $refLocalStorage ) {
         eval '$refLocalStorage = new '
           . $localStorage
           . '($localStorageOptions);';
     }
-    $class->lmLog( "Local cache initialization failed: $@", 'error' )
+    $class->lmLog("Local cache initialization failed: $@",'error')
       unless ( defined $refLocalStorage );
     return DECLINED;
 }
